from typing import Dict, Set
from rest_framework.request import Request
from rest_framework.permissions import BasePermission


def check_client_roles(roles_to_verify: Set, resource_access: Dict):
    _client_roles = set(roles_to_verify)
    for client_id, roles_obj in resource_access.items():
        _client_roles = _client_roles - {f"{client_id}:{role}" for role in (roles_obj.get("roles", []))}
    return not bool(_client_roles)


def check_realm_roles(roles_to_verify: Set, realm_access: Dict):
    _realm_roles = set(roles_to_verify)

    realm_roles = set(realm_access.get("roles", []))
    _realm_roles = _realm_roles - realm_roles
    return not bool(_realm_roles)


class RolePermission(BasePermission):
    has_realm_roles = []
    has_client_roles = []

    def has_permission(self, request: Request, view):
        token_info = request.auth

        has_client_roles_permissions = check_client_roles(self.has_client_roles, token_info.get("resource_access", {}))
        has_realm_roles_permissions = check_realm_roles(self.has_realm_roles, token_info.get("realm_access", {}))

        return has_client_roles_permissions and has_realm_roles_permissions



class ServiceAccountPermission(RolePermission):

    def has_permission(self, request: Request, view):
        token_info = request.auth

        if "clientId" not in token_info:
            return False

        if not token_info["preferred_username"].startswith("service-account"):
            return False
        
        return super().has_permission(request, view)
        


#!/usr/bin/env python

"""Tests for `keycloak_django_utils` package."""


import unittest

from keycloak_django_utils import keycloak_django_utils


class TestKeycloak_django_utils(unittest.TestCase):
    """Tests for `keycloak_django_utils` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""



from rest_framework.authentication import TokenAuthentication
from rest_framework.exceptions import APIException, status

from keycloak_django_utils.jwt import validate_token


class AuthenticationFailed(APIException):
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = "Incorrect authentication credentials."
    default_code = "authentication_failed"


class BearerAuthentication(TokenAuthentication):
    keyword = "Bearer"

    def authenticate_credentials(self, key):
        try:
            validated_token_info = validate_token(key)
        except Exception as e:
            raise AuthenticationFailed(detail=str(e))
        return None, validated_token_info



import json
from dataclasses import dataclass, fields

import requests
import jwt

from django.core.cache import cache
from django.conf import settings


@dataclass
class TokenHeader:
    alg: str = None
    typ: str = None
    kid: str = None


def get_keycloak_public_key(key_id: str):
    json_key = cache.get(key_id)

    if not json_key:
        url = settings.KEYCLOAK_JWKS_ENDPOINT

        response = requests.get(url)
        response.raise_for_status()
        data = response.json()
        json_key = json.dumps(
            next(filter(lambda item: item["kid"] == key_id, data.get("keys", [])), None)
        )
        cache.set(key_id, json_key)

    return jwt.algorithms.RSAAlgorithm.from_jwk(json_key)


def get_token_header(access_token: str):
    header = jwt.get_unverified_header(access_token)
    field_names = [i.name for i in fields(TokenHeader)]
    return TokenHeader(**{k: v for k, v in header.items() if k in field_names})


def validate_token(access_token: str, audience="account"):
    token_header = get_token_header(access_token)
    public_key = get_keycloak_public_key(token_header.kid)
    return jwt.decode(
        access_token, key=public_key, algorithms=token_header.alg, audience=audience
    )
